import path from "path";
import GppCompiler from "../Compiler/GppCompiler";
import printDebug from "../Tools/DebugPrint";
import setGetDefaultCompiler from "../Tools/SetGetDefaultCompiler";
import HaveCompiler from "./interface/HaveCompiler";
import Optimize from "./interface/Optimize";
import SourceFiles from "./interface/SourceFiles";
import Project from "./Project";
import { getNmakePath } from "../Tools/NmakePath";
import { getNmakeDir } from "../Tools/NmakeDir";

export default class CppProject implements Project , SourceFiles,HaveCompiler,Optimize {
    name: string;
    compiler: GppCompiler;
    sourceFilesPath: string[]=[];
    optimize: "fast" | "fastest" | "normal" = "fast"
    typeName: string = "cpp";
    addFiles = (...files: string[]):CppProject =>{
        printDebug("adding source files "+files)
        files.forEach(v=>{
            this.sourceFilesPath.push(
                path.join(getNmakeDir(),v)
            )
        });
        printDebug("all files in "+this.name+" :"+this.sourceFilesPath)
        return this
    };

    constructor(name:string)
    constructor(name:string,compiler:GppCompiler)

    constructor(name:string,compiler?:GppCompiler){
        printDebug(`adding new ${this.typeName} project`)
        if(compiler){
            this.compiler = compiler
        }else{
            this.compiler = setGetDefaultCompiler("g++",()=>{return new GppCompiler()})
        }
        this.name=name
    }
}